      Subroutine MyModel1 ( IDTask, iMod, IsUndr,
     *                      iStep, iTer, iEl, Int,
     *                      X, Y, Z,
     *                      Time0, dTime,
     *                      Props, Sig0, Swp0, StVar0,
     *                      dEps, D, BulkW,
     *                      Sig, Swp, StVar, ipl,
     *                      nStat,
     *                      NonSym, iStrsDep, iTimeDep, iTang,
     *                      iAbort )
!
! Purpose: User supplied soil model (Example: Drucker-Prager)
!  Depending on IDTask, 1 : Initialize state variables
!                       2 : calculate stresses,
!                       3 : calculate material stiffness matrix
!                       4 : return number of state variables
!                       5 : inquire matrix properties
!                       6 : calculate elastic material stiffness matrix
!                           return switch for non-symmetric D-matrix
!                           stress/time dependent matrix
! Arguments:
!          I/O  Type
!  IDTask   I   I    : see above
!  iMod     I   I    : model number (1..10)
!  IsUndr   I   I    : =1 for undrained, 0 otherwise
!  iStep    I   I    : Global step number
!  iter     I   I    : Global iteration number
!  iel      I   I    : Global element number
!  Int      I   I    : Global integration point number
!  X        I   R    : X-Position of integration point
!  Y        I   R    : Y-Position of integration point
!  Z        I   R    : Z-Position of integration point
!  Time0    I   R    : Time at start of step
!  dTime    I   R    : Time increment
!  Props    I   R()  : List with model parameters
!  Sig0     I   R()  : Stresses at start of step
!  Swp0     I   R    : Excess pore pressure start of step
!  StVar0   I   R()  : State variable at start of step
!  dEps     I   R()  : Strain increment
!  D       I/O  R(,) : Material stiffness matrix
!  BulkW   I/O  R    : Bulkmodulus for water (undrained only)
!  Sig      O   R()  : Resulting stresses
!  Swp      O   R    : Resulting excess pore pressure
!  StVar    O   R()  : Resulting values state variables
!  ipl      O   I    : Plasticity indicator
!  nStat    O   I    : Number of state variables
!  NonSym   O   I    : Non-Symmetric D-matrix ?
!  iStrsDep O   I    : =1 for stress dependent D-matrix
!  iTimeDep O   I    : =1 for time dependent D-matrix
!  iAbort   O   I    : =1 to force stopping of calculation
!
      Implicit Double Precision (A-H, O-Z)
!
      Dimension Props(*), Sig0(*), StVar0(*), dEps(*), D(6,6),
     *          Sig(*),   StVar(*)
!
!---  Local variables
!
      Dimension dSig(6), xN1(3), xN2(3), xN3(3)

!
! Expected contents of Props(1..?)
!  1 : G    Shear modulus
!  2 : xNu  Poissons' ratio
!  3 : Alf  Steepness of yield surface in p-q-plane
!  4 : Bet  Steepness of plastic potential function in p-q-plane
!  5 : C    'cohesion' term f = q + Alf*p - C
! 10 : BFac Ratio K/G to calculate Bulk modulus

      nStatV = 0
      If (IDTask .Eq. 1) Then ! Initialize state variables
        ! When there are state variables one could check a maximum
        ! For instance:
        !   StVar(1) contains maximum isotropic stress
        !   StVar(2) contains maximum shear mobilisation
        ! p = -(Sig0(1)+Sig0(2)+Sig0(3))/3
        ! or Call PrnSig(Sig0, ...., P0,Q0)
        !    P = -P0 ! compression positive
        ! ratio = Q0/Max(P,1d0) ! mobilisation degree
        ! StVar0(1) = Max (StVar0(1), p )
        ! StVar0(2) = Max (StVar0(2), ratio )
        G   = Props(1) ! G
        xNu = Props(2) ! nu

        If (xNu.Gt.0.5d0-1d-6 .Or. xNu.Lt.-1d-6) Then
          Call OK_MessageBox(' Invalid value for "nu" (param 2)')
          iAbort = 1
          Return
        End If
        if (xNu.Lt.0) then
          xNu = 0.0
          Props(2) = xNu
        end if
        if (xNu.Gt.0.495d0) then
          xNu = 0.495
          Props(2) = xNu
        end if
        BFac= (2+2*xNu)/(3-6*xNu) ! Bulk modulus/G
        Props(10) = BFac
        Bulk= BFac * G

        If (iel+int.eq.2) Call WriVec(1,'Props',Props,10)

      End If  ! IDTask = 1

      If (IDTask .Eq. 2) Then ! Calculate stresses
        Call CopyRVec( StVar0, StVar, nStatV )
        If (IsUndr.Eq.1) Then
          dEpsV = dEps(1) + dEps(2) + dEps(3)
          dSwp  = BulkW * dEpsV
          Swp   = Swp0 + dSwp
        Else
          Swp = Swp0
        End If
        If (iEl+Int+iter.Eq.3 .And. iStep.Lt.10) Then
          Call WriMat( -1, 'D66', D, 6, 6, 6 )
        End If
        Call MatVec( D, 6, dEps, 6, dSig)
        Call AddVec( Sig0, dSig, 1d0, 1d0, 6, Sig )

        iOpt = 1
        Call WriVec( -1, 'Sig', Sig, 6)
        Call PrnSig(iOpt, Sig, xN1, xN2, xN3, S1, S2, S3, P, Q)

        ipl = 0
        G    = Props(1)
        xNu  = Props(2) ! nu
        Alf  = Props(3)
        Bet  = Props(4)
        C    = Props(5)

        Alf  = Max( Alf, 1d-10 ) ! prevent later division by zero
        P0   = - ( Sig0(1) + Sig0(2) + Sig0(3) ) /3
        P0   = 100
        G    = P0 / 100 * Props(1)
        Bulk = Props(10) * G
        f = q + Alf * p - c

        If (f .Gt. 1d-6) Then ! plastic stress correction
          !  f = q + Alf * p - c
          !  p = pe - xLam * Bulk * Bet
          !  q = qe - xLam * 3*G
          pe = p
          qe = q
          dfdp = Alf
          dfdq = 1
          dpdl = -Bulk*Bet
          dqdl = -3*G
          dfdl = (dfdp*dpdl + dfdq*dqdl)
          xLam = -f / dfdl
          p = pe - xLam * Bulk * Bet
          q = qe - xLam * 3 * G
          f = q + Alf * p - c
          ipl = 1
          If (p.Gt.c/Alf) Then ! apex
            p = c/Alf
            q = 0
            ipl = 2
          End If
          If (qe.Gt.1d-10) Then     !
            s1 =  p + q/qe*(s1-pe)
            s2 =  p + q/qe*(s2-pe)
            s3 =  p + q/qe*(s3-pe)
          Else
            s1 =  p
            s2 =  p
            s3 =  p
          End If
        End If
        ! as example: update state variables
        ! Pt = -P ! compression positive
        ! ratio = Q/Max(Pt,1d0) ! mobilisation degree
        ! StVar(1) = Max (StVar0(1), pt )
        ! StVar(2) = Max (StVar0(2), ratio )
        !


        ! back to Cartesian stresses
        Call CarSig(S1,S2,S3,xN1,xN2,xN3,Sig)

        If (ipl.Gt.0) Then ! just checking f~0
          iOpt = 0
          Call PrnSig(iOpt,Sig,xN1,xN2,xN3,S1,S2,S3,P,Q)
          f = q + Alf * p - c
        End If
      End If ! IDTask = 2

      If ( IDTask .Eq. 3 .Or.
     *     IDTask .Eq. 6     ) Then ! Calculate D-Matrix
        G   = Props(1) ! G
        P0  = - ( Sig0(1) + Sig0(2) + Sig0(3) ) /3
        P0  = 100
        G   = P0 / 100 * Props(1)
        xNu = Props(2) ! nu

        F1  = 2*G*(1-xNu)/(1-2*xNu)
        F2  = 2*G*( xNu )/(1-2*xNu)
        Call MZeroR(D,36)
        Do i=1,3
          Do j=1,3
            D(i,j) = F2
          End Do
          D(i,i) = F1
          D(i+3,i+3) = G
        End Do
!        Call WriMat( 1, 'D_Usr', D, 6, 6, 6 )
        BulkW = 0
        If (IsUndr.Eq.1) Then
          ! BulkW = ...
          xNu_U = 0.495d0
          Fac=(1+xNu_U)/(1-2*xNu_U) - (1+xNu)/(1-2*xNu)
          Fac=2D0*G/3D0  * Fac
          BulkW = Fac
        End If
      End If  ! IDTask = 3, 6

      If (IDTask .Eq. 4) Then ! Number of state parameters
        nStat    = nStatV
      End If  ! IDTask = 4

      If (IDTask .Eq. 5) Then ! matrix type
        NonSym   = 0  ! 1 for non-symmetric D-matrix
        iStrsDep = 0  ! 1 for stress dependent D-matrix
        iTang    = 0  ! 1 for tangent D-matrix
        iTimeDep = 0  ! 1 for time dependent D-matrix
      End If  ! IDTask = 5

      Return
      End ! MyModel1
